#pragma once

#include "Actor.h"
#include "ManagerInterface.h"
#include "WASActor.h"

/////////////////////////////////////////////
#include "incconferenceintf.h"
#include "INpMsgrP2P.h"
/////////////////////////////////////////////

#ifdef __USE_PROFILE_IMG_BY_P2P__
#include "ProfileImageSender.h"
#include "ProfileImageRecverP2P.h"
class CProfileImageRecverP2P;
#endif

class CProfileImageActor : 	public CActor
	, public IGTProfileImageActor
	, public IGTWASActorEventPrivate
#ifdef __USE_PROFILE_IMG_BY_P2P__
    , public INpServicePlugin
    , public _INpP2PMngr_IncomingP2PSessionHandler
#endif
{
public:
	CProfileImageActor(CMessengerActor *pActor);
	virtual ~CProfileImageActor(void);

    virtual void SetProfileImageProxy(IGTProfileImageProxy *proxy)
    {
        m_pProfileImageProxy = proxy;
        
    }
    
	static DWORD GetFileSize(LPCTSTR pszFileName, LPDWORD lpFileSizeHigh);
	static DWORD GetFileCRC(LPCTSTR pszFileName, DWORD * pdwCRC);
	static DWORD GetCRC(DWORD nCRC_Accum, char * pData,int nSize);
	static WSTRING GetProfileImageFilePath(const WSTRING &strUserID, const WSTRING &strFileName);
	static WSTRING GetProfileImageFilePathBySizeType(LPCWSTR strUserID, GTProfileImageSizeType eSizeType);
	// For GTObject
protected:	
	virtual void OnReleaseObject(void) { delete this; }
	

	// For API..
public:
#ifdef __USE_PROFILE_IMG_BY_P2P__
    virtual bool RequestProfileImageByP2P(LPCWSTR strUserID, GTProfileImageSizeType eSizeType = profileOriginalSize, DWORD dwUserData1 = 0, DWORD dwUserData2 = 0, bool bForceMode = false, trid_t *pTrid = NULL);
#endif

	virtual bool RequestProfileImage(LPCWSTR strUserID, GTProfileImageSizeType eSizeType = profileOriginalSize, DWORD dwUserData1 = 0, DWORD dwUserData2 = 0, bool bForceMode = false, trid_t *pTrid = NULL);
    virtual bool RequestSetMyProfileImage(LPCWSTR pszFileName, trid_t *pTrid = NULL);

    virtual GTProfileImageCheckResult GetUserProfileImagePath(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, size_t &nBufferSize/*OUT*/, char *szFilePath /*OUT*/);
	
	virtual void StopSetMyProfileImage(void);

	// For Internal Uses..
public:

	CMessengerActor *GetMessengerActor(void) { return m_pMessengerActor; }

    IGTProfileImageProxy *GetProfileImageProxy(void) { return m_pProfileImageProxy; }
	
#ifdef __USE_PROFILE_IMG_BY_P2P__	
	INpP2PMngr* GetNpP2PMngr(void)
	{
		return m_pNpP2PMngr;
	}

	// ProfileImageRecver 
	void OnProfileImageError(CProfileImageRecverP2P *pImage, GTWASCallErrorCode eWASError, GTProfileImgError ePIError);
	void OnProfileImageReady(CProfileImageRecverP2P *pImage);
#endif
    
    bool IsReusableProfileImage(LPCWSTR strUserId, GTProfileImageSizeType eSizeType, unsigned int *pnCRC);
    bool UpdateProfileImageCRC(LPCWSTR strUserId, DWORD dwProfileImgCrc);
    bool SaveCurrentProfileImageCRC(LPCWSTR strUserId, DWORD dwProfileImgCrc, GTProfileImageSizeType eSizeType);

private:

    GTProfileImageCheckResult _CheckImage(LPCWSTR strUserID, LPCWSTR strFilePath,  GTProfileImageSizeType eSizeType, DWORD &dwCRC);
    WSTRING _GetUserProfileImagePath(LPCWSTR strUserID, GTProfileImageSizeType eSizeType);
	GTProfileImageCheckResult _GetUserProfileImagePathEx(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, DWORD &dwCRC/*OUT*/, size_t &nBufferSize/*OUT*/, char *szFilePath /*OUT*/);

    bool _OnResponseSuccUploadImage(const GT_PROFILE_IMAGE_DATA& sData);
    bool _OnResponseSuccDownloadImage(const GT_PROFILE_IMAGE_DATA& sData);
        
	void _OnSuccessGetUserImageOK(LPCWSTR strUserID, LPCWSTR strFilePath);

	bool _HasTransaction(trid_t trid)
	{
		mytridmap_t::iterator it = m_mapMyTrid.find(trid);
		if(it != m_mapMyTrid.end())
		{
			m_mapMyTrid.erase(it);
			return true;
		}
		return false;
	}
	void _AddTransaction(trid_t trid)
	{
		m_mapMyTrid.insert(mytridmap_t::value_type(trid, trid));
	}

#ifdef __USE_PROFILE_IMG_BY_P2P__
	CProfileImageSender* _CreateProfileImageSender();
	void _EraseProfileImageSender(CProfileImageSender *pSender);
	void DeleteSender(CProfileImageSender *pSender);
	void DeleteRecver(LPCWSTR strUserID);

	// INpServicePlugin 
	virtual void SetOFFLINE();
	virtual void Delete();
	virtual void * GetServiceAppInterfacePointer() { return (void *)this; }
	virtual void OnChange_MyContactAddr(LPCSTR /*szTcpCAddr*/, LPCSTR /*szUdpCAddr*/) {}
#endif


protected:

#ifdef __USE_PROFILE_IMG_BY_P2P__
	// _INpP2PMngr_IncomingP2PSessionHandler 
	virtual bool CALLBACK NpP2PMngr_OnIncoming_NpStreamSession(
		INpStreamSession * pIncomingSession, 
		LPCWSTR wszFromServiceId,
		INpPacket * pPacket,
		NPP2PSESSION_PROTOCOL_TYPE ePType,
		bool bIsRelay,
		bool bIsDual);

	virtual bool CALLBACK NpP2PMngr_OnIncoming_NpRealTimeSession(
		INpRealTimeSession * /*pIncomingSession*/, 
		LPCWSTR /*wszFromServiceId*/, 
		USHORT /*usApplicationLayerCMMD*/, 
		ULONG /*ulApplicationLayerParam*/, 
		USHORT /*uDataLen*/, 
		LPCSTR /*szData*/,
		NPP2PSESSION_PROTOCOL_TYPE /*ePType*/,
		bool /*bIsRelay*/,
		bool /*bIsDual*/){ return false; }

	virtual bool CALLBACK NpP2PMngr_IsAvailableP2P(LPCWSTR wszFromServiceId, ULONG ulApplicationLayerParam) { return false; }
#endif

	// IGTWASActorEvent 
    bool OnGTWASActor_ResponseProfileImage(IGTWASActor *pActor, 
                                           trid_t trid,
                                           GTWASCallErrorCode eErrorCode, 
                                           GTProfileImgError eProfileError, 
                                           bool bUploadFile, 
                                           const GT_PROFILE_IMAGE_DATA& sData,
                                           DWORD dwUserData1, 
                                           DWORD dwUserData2);
private:


#ifdef __USE_PROFILE_IMG_BY_P2P__
	typedef std::map<CProfileImageSender *, CProfileImageSender *> profileimgsendermap_t;
	typedef std::map<WSTRING, CProfileImageRecverP2P *> profileimgrecvermap_t;

	profileimgsendermap_t m_mapProfileImageSender;
	profileimgrecvermap_t m_mapProfileImageRecver;
	INpP2PMngr * m_pNpP2PMngr;

	friend class CProfileImageRecverP2P;
	friend class CProfileImageSender;
#endif

	bool m_bUploadingProfileImage;

	trid_t m_tridSetProfileImage;
	trid_t m_tridDownProfileImage;

	// WAS TRANSACTION ID =>  Actor  WAS  üũϱ .
	mytridmap_t m_mapMyTrid;

	CMessengerActor *m_pMessengerActor;
	    
    IGTProfileImageProxy *m_pProfileImageProxy;
};
